(*
*=================================================================================
* Unit for base module for Popbill API SDK. It include base functionality for
* RESTful web service request and parse json result. It uses Linkhub module
* to accomplish authentication APIs.
*
* http://www.popbill.com
* Author : Jeong Yohan (code@linkhubcorp.com)
* Written : 2018-02-26
* Updated : 2025-02-24
* Thanks for your interest.
*=================================================================================
*)

unit PopbillKakao;

interface

uses
        TypInfo,SysUtils,Classes,Popbill,Linkhub;
type
        // ATS-˸, FTS-ģ, FMS-ģ ̹
        EnumKakaoType = (ATS,FTS,FMS);

        // īī ä 
        TPlusFriendInfo = class
        public
                plusFriendID : string;
                plusFriendName : string;
                regDT : string;
                state : string;
                stateDT : string;
        end;

        TPlusFriendList = Array of TPlusFriendInfo;

        // 
        TKakaoChargeInfo = class
        public
                unitCost : string;
                chargeMethod : string;
                rateSystem : string;
        end;


        // ߽Źȣ
        TKakaoSenderNumber = class
        public
            number : string;
            state : integer;
            representYN : Boolean;
        end;

        TKakaoSenderNumberList = Array of TKakaoSenderNumber;


        // ۿû - ģ ư 
        TSendKakaoButton = class
        public
                buttonName : string;
                buttonType : string;
                buttonURL1 : string;
                buttonURL2 : string;
                buttonTG : string;
        end;

        TSendKakaoButtonList = Array Of TSendKakaoButton;


        // ۿû - /޽ 
        TSendKakaoReceiver = class
        public
                rcv : string;
                rcvnm : string;
                msg : string;
                altsjt : string;
                altmsg : string;
                interOPRefKey : string;
                buttonList : TSendKakaoButtonList;
        end;

        
        TSendKakaoReceiverList = Array Of TSendKakaoReceiver;

        // ۰  Detail 迭
        TSentKakaoDetail = class
        public
                state : integer;
                sendDT : string;
                result : integer;
                resultDT : string;
                contentType : string;
                receiveNum : string;
                receiveName : string;
                content : string;
                altContentType : integer;
                altSendDT : string;
                altResult : integer;
                altResultDT : string;
                altSubject : string;
                altContent : string;
                receiptNum : String;
                requestNum : String;
                interOPRefKey : String;
        end;
        TSentKakaoDetailList = Array Of TSentKakaoDetail;

         // ۰ GetMessage
        TSentKakaoInfo = class
        public
                contentType : string;
                templateCode : string;
                plusFriendID : string;
                sendNum : string;
                altSubject : string;
                altContent : string;
                altSendType : string;
                reserveDT : string;
                adsYN : Boolean;
                imageURL : string;
                sendCnt : string;
                successCnt : string;
                failCnt : string;
                altCnt : string;
                cancelCnt : string;
                msgs : TSentKakaoDetailList;
                btns : TSendKakaoButtonList;
        end;


        // ȸ Search
        TKakaoSearchList = class
        public
                code : Integer;
                total : Integer;
                perPage : Integer;
                pageNum : Integer;
                pageCount : Integer;
                message : string;
                list : TSentKakaoDetailList;
        end;

        // ø 
        TATSTemplate = class
        public
                templateCode : string;
                templateName : String;
                template : string;
                plusFriendID : string;
                ads : string;
                appendix : string;
                secureYN : Boolean;
                state : Integer;
                stateDT : string;
                btns : TSendKakaoButtonList;
        end;

        TATSTemplateList = Array of TATSTemplate;

        TKakaoService = class(TpopbillBaseService)
        public
                constructor Create(LinkID : String; SecretKey : String);

                // GetURL
                function GetURL(CorpNum : String; TOGO: String; UserID: String = '') : String;

                // īī ä  ˾ URL
                function GetPlusFriendMgtURL(CorpNum : String; UserID: String) : String;

                // ߽Źȣ  ˾ URL
                function GetSenderNumberMgtURL(CorpNum : String; UserID: String) : String;

                // ˸ ø ˾ URL
                function GetATSTemplateMgtURL(CorpNum : String; UserID: String) : String;

                // īī ۳ ˾ URL
                function GetSentListURL(CorpNum : String; UserID: String) : String;

                // īī ä  Ȯ
                function ListPlusFriendID(CorpNum : String; UserID : String = '') : TPlusFriendList;

                // ߽Źȣ  Ȯ
                function GetSenderNumberList(CorpNum: String; UserID : String = '') : TKakaoSenderNumberList;

                // ˸ ø  Ȯ
                function GetATSTemplate(CorpNum : String; templateCode : String; UserID : String = '') : TATSTemplate;
                
                // ˸ ø  Ȯ
                function ListATSTemplate(CorpNum : String; UserID : String = '') : TATSTemplateList;

                // ߽Źȣ Ͽ Ȯ
                function CheckSenderNumber(CorpNum : String; SenderNumber : String; UserID : String = '') : TResponse;


                // ˸ (ܰ)
                function SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; AltSendType : String; ReserveDT: String; ReceiverNum : String; ReceiverName : String; ATSMsg : String; ATSAltMsg : String; UserID : String = ''; requestNum : String = '') : String; overload;

                // ˸ (ܰ) - ư ߰                
                function SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; AltSendType : String; ReserveDT: String; ReceiverNum : String; ReceiverName : String; ATSMsg : String; ATSAltMsg : String; Buttons : TSendKakaoButtonList; UserID : String = ''; requestNum : String = '') : String; overload;

                // ˸ (ܰ) - ü  ߰
                function SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; AltSendType : String; ReserveDT: String; ReceiverNum : String; ReceiverName : String; ATSMsg : String; ATSAltSjt : String; ATSAltMsg : String; Buttons : TSendKakaoButtonList; UserID : String = ''; requestNum : String = '') : String; overload;

                // ˸ (뷮)
                function SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; Content : String; AltContent: String; AltSendType : String; ReserveDT: String; Receivers : TSendKakaoReceiverList; UserID : String = ''; requestNum : String = '') : String; overload;

                // ˸ (뷮) - ư ߰                
                function SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; Content : String; AltContent: String; AltSendType : String; ReserveDT: String; Receivers : TSendKakaoReceiverList; Buttons : TSendKakaoButtonList; UserID : String = ''; requestNum : String = '') : String; overload;                

                // ˸ (뷮)- ü  ߰
                function SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; Content : String; AltSubject: String; AltContent: String; AltSendType : String; ReserveDT: String; Receivers : TSendKakaoReceiverList; Buttons : TSendKakaoButtonList; UserID : String = ''; requestNum : String = '') : String; overload;                


                // ģ ؽƮ (ܰ)
                function SendFTS(CorpNum : String; PlusFriendID : String; SenderNum : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; ReceiverNum : String; ReceiverName : String; FTSMsg : String; FTSAltMsg : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String; overload;

                // ģ ؽƮ (ܰ) - ü  ߰
                function SendFTS(CorpNum : String; PlusFriendID : String; SenderNum : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; ReceiverNum : String; ReceiverName : String; FTSMsg : String; FTSAltSjt : String; FTSAltMsg : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String; overload;

                // ģ ؽƮ  (뷮)
                function SendFTS(CorpNum : String; PlusFriendID : String; SenderNum : String; Content : String; AltContent : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; Receivers : TSendKakaoReceiverList; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String; overload;

                // ģ ؽƮ  (뷮) - ü  ߰
                function SendFTS(CorpNum : String; PlusFriendID : String; SenderNum : String; Content : String; AltSubject : String; AltContent : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; Receivers : TSendKakaoReceiverList; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String; overload;


                // ģ ̹ (ܰ)
                function SendFMS(CorpNum : String; PlusFriendID : String; SenderNum : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; ReceiverNum : String; ReceiverName : String; FTSMsg : String; FTSAltMsg : String; FMSFilePath : String; ImageURL : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String; overload;

                // ģ ̹ (ܰ) - ü  ߰
                function SendFMS(CorpNum : String; PlusFriendID : String; SenderNum : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; ReceiverNum : String; ReceiverName : String; FTSMsg : String; FTSAltSjt : String; FTSAltMsg : String; FMSFilePath : String; ImageURL : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String; overload;

                // ģ ̹ (뷮)
                function SendFMS(CorpNum : String; PlusFriendID : String; SenderNum : String; Content : String; AltContent : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; Receivers : TSendKakaoReceiverList; FMSFilePath : String; ImageURL : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String; overload;

                // ģ ̹ (뷮) - ü  ߰
                function SendFMS(CorpNum : String; PlusFriendID : String; SenderNum : String; Content : String; AltSubject : String; AltContent : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; Receivers : TSendKakaoReceiverList; FMSFilePath : String; ImageURL : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String; overload;



                // 
                function CancelReserve(CorpNum : String; receiptNum : String; UserID : String = '') : TResponse;

                //  - ûȣ Ҵ
                function CancelReserveRN(CorpNum : String; requestNum : String; UserID : String = '') : TResponse;

                // 󼼳  ۻ Ȯ
                function GetMessages(CorpNum : String; receiptNum : String; UserID : String = '') : TSentKakaoInfo;

                // 󼼳  ۻ Ȯ - ûȣ Ҵ
                function GetMessagesRN(CorpNum : String; requestNum : String; UserID : String = '') : TSentKakaoInfo;

                // ȸ
                function Search(CorpNum : String; SDate: String; EDate : String; State : Array Of String; Item : Array Of String; ReserveYN : String; SenderYN : boolean; Page : Integer; PerPage : Integer; Order : String; UserID : String = ''): TKakaoSearchList; overload;
                function Search(CorpNum : String; SDate: String; EDate : String; State : Array Of String; Item : Array Of String; ReserveYN : String; SenderYN : boolean; Page : Integer; PerPage : Integer; Order : String; QString : String; UserID : String): TKakaoSearchList; overload;

                // ȸ ۴ܰ Ȯ
                function GetUnitCost(CorpNum : String; KakaoMsgType:EnumKakaoType; UserID : String = '') : Single;

                //  Ȯ
                function GetChargeInfo(CorpNum : String; KakaoMsgType:EnumKakaoType; UserID : String = '') : TKakaoChargeInfo;



        end;
implementation

constructor TKakaoService.Create(LinkID : String; SecretKey : String);
begin
        inherited Create(LinkID,SecretKey);
        AddScope('153');
        AddScope('154');
        AddScope('155');
end;

function BoolToStr(b:Boolean):String;
begin
        if b = true then BoolToStr:='True';
        if b = false then BoolToStr:='False';
end;

// ˾ URL
function TKakaoService.GetURL(CorpNum : String; TOGO : String; UserID : String) : string;
var
        responseJson : String;
begin

        try
                // ߽Źȣ  ˾
                if TOGO = 'SENDER' then
                        responseJson := httpget('/Message/?TG='+TOGO, CorpNum, UserID)
                else
                        responseJson := httpget('/KakaoTalk/?TG='+TOGO, CorpNum, UserID);

                result := getJsonString(responseJson, 'url');
        except
                on le : EPopbillexception do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.message);
                        end;

                end;
        end;
end;

// īī ä  ˾ URL
function TKakaoService.GetPlusFriendMgtURL(CorpNum : String; UserID : String) : string;
var
        responseJson : String;
begin
        try
                responseJson := httpget('/KakaoTalk/?TG=PLUSFRIEND', CorpNum, UserID);
                result := getJsonString(responseJson, 'url');
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.message);
                                exit;
                        end;
                end;
        end;
end;

// ߽Źȣ  ˾ URL
function TKakaoService.GetSenderNumberMgtURL(CorpNum : String; UserID : String) : string;
var
        responseJson : String;
begin
        try
                responseJson := httpget('/Message/?TG=SENDER', CorpNum, UserID);
                result := getJsonString(responseJson, 'url');
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.message);
                                exit;
                        end;
                end;
        end;
end;

// ˸ ø  ˾ URL
function TKakaoService.GetATSTemplateMgtURL(CorpNum : String; UserID : String) : string;
var
        responseJson : String;
begin
        try
                responseJson := httpget('/KakaoTalk/?TG=TEMPLATE', CorpNum, UserID);
                result := getJsonString(responseJson, 'url');
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.message);
                                exit;
                        end;
                end;
        end;
end;

// īī ۳ ˾ URL
function TKakaoService.GetSentListURL(CorpNum : String; UserID : String) : string;
var
        responseJson : String;
begin
        try
                responseJson := httpget('/KakaoTalk/?TG=BOX', CorpNum, UserID);
                result := getJsonString(responseJson, 'url');
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.message);
                                exit;
                        end;
                end;
        end;
end;

// īī ä  Ȯ
function TKakaoService.ListPlusFriendID(CorpNum : String; UserID : String = '') : TPlusFriendList;
var
        responseJson : String;
        jSons : ArrayOfString;
        i : Integer;
begin
        try
                responseJson := httpget('/KakaoTalk/ListPlusFriendID',CorpNum, UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code,le.message);
                                exit;
                        end
                        else
                        begin
                                SetLength(result,0);
                                SetLength(jSons,0);
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                exit;
        end
        else
        begin
                try
                        jSons := ParseJsonList(responseJson);
                        SetLength(result,Length(jSons));

                        for i:=0 to Length(jSons)-1 do
                        begin
                                result[i] := TPlusFriendInfo.Create;
                                result[i].plusFriendID := getJsonString(jSons[i], 'plusFriendID');
                                result[i].plusFriendName := getJsonString(jSons[i], 'plusFriendName');
                                result[i].regDT := getJsonString(jSons[i], 'regDT');
                                result[i].state := getJsonString(jSons[i], 'state');
                                result[i].stateDT := getJsonString(jSons[i], 'stateDT');
                        end;
                except
                        on E:Exception do begin
                                if FIsThrowException then
                                begin
                                        raise EPopbillException.Create(-99999999,'ó .[Malformed Json]');
                                        exit;
                                end
                                else
                                begin
                                        SetLength(result,0);
                                        SetLength(jSons,0);
                                        setLastErrCode(-99999999);
                                        setLastErrMessage('ó .[Malformed Json]');
                                        exit;
                                end;
                        end;
                end;
        end;
end;

// ߽Źȣ  ȸ
function TKakaoService.GetSenderNumberList(CorpNum : String; UserID: String = '') : TKakaoSenderNumberList;
var
        responseJson : String;
        jSons : ArrayOfString;
        i : Integer;
begin


        try
                responseJson := httpget('/Message/SenderNumber',CorpNum, UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.message);
                                exit;
                        end
                        else
                        begin
                                SetLength(result,0);
                                SetLength(jSons,0);
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                exit;
        end
        else
        begin
                try
                        jSons := ParseJsonList(responseJson);
                        SetLength(result,Length(jSons));

                        for i:= 0 to Length(jSons)-1 do
                        begin
                                result[i] := TKakaoSenderNumber.Create;
                                result[i].number := getJsonString(jSons[i],'number');
                                result[i].state := getJsonInteger(jSons[i],'state');
                                result[i].representYN := getJsonBoolean(jSons[i],'representYN');
                        end;
                except
                        on E:Exception do begin
                                if FIsThrowException then
                                begin
                                        raise EPopbillException.Create(-99999999,'ó .[Malformed Json]');
                                        exit;
                                end
                                else
                                begin
                                        SetLength(result,0);
                                        SetLength(jSons,0);
                                        setLastErrCode(-99999999);
                                        setLastErrMessage('ó .[Malformed Json]');
                                        exit;
                                end;
                        end;
                end;
        end;
end;

// ˸ ø Ȯ
function TKakaoService.GetATSTemplate(CorpNum : String; templateCode : String; UserID : String = '') : TATSTemplate;
var
        responseJson : String;
        btnjSons : ArrayOfString;
        i : Integer;

begin
        if templateCode = '' then
        begin
                  if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999,'øڵ尡 Էµ ʾҽϴ');
                        exit;
                end
                else
                begin
                        result := TATSTemplate.Create;
                        setLastErrCode(-99999999);
                        setLastErrMessage('øڵ尡 Էµ ʾҽϴ');
                        exit;
                end;
        end;
        try
                responseJson := httpget('/KakaoTalk/GetATSTemplate/'+templateCode, CorpNum, UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code,le.message);
                                exit;
                        end
                        else
                        begin
                                result := TATSTemplate.Create;
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                exit;
        end
        else
        begin

                try
                        result := TATSTemplate.Create;
                        result.templateCode := getJsonString(responseJson, 'templateCode');
                        result.templateName := getJsonString(responseJson, 'templateName');
                        result.template := getJsonString(responseJson, 'template');
                        result.plusFriendID := getJsonString(responseJson, 'plusFriendID');
                        result.ads := getJsonString(responseJson, 'ads');
                        result.appendix := getJsonString(responseJson, 'appendix');
                        result.secureYN := getJsonBoolean(responseJson, 'secureYN');
                        result.state := StrtoInt(getJsonString(responseJson, 'state'));
                        result.stateDT := getJsonString(responseJson, 'stateDT');

                        // ˸ ø ư
                        btnjSons := getJSonList(responseJson, 'btns');
                        SetLength(result.btns, Length(btnjSons));

                        for i:= 0 to Length(btnjSons)-1 do
                        begin
                                result.btns[i] := TSendKakaoButton.Create;
                                result.btns[i].buttonName := getJSonString(btnjSons[i],'n');
                                result.btns[i].buttonType := getJSonString(btnjSons[i],'t');
                                result.btns[i].buttonURL1 := getJSonString(btnjSons[i],'u1');
                                result.btns[i].buttonURL2 := getJSonString(btnjSons[i],'u2');
                        end;
                        SetLength(btnjSons,0);
                except
                        on E:Exception do begin
                                if FIsThrowException then
                                begin
                                        raise EPopbillException.Create(-99999999,'ó .[Malformed Json]');
                                        exit;
                                end
                                else
                                begin
                                        result := TATSTemplate.Create();
                                        setLastErrCode(-99999999);
                                        setLastErrMessage('ó .[Malformed Json]');
                                        exit;
                                end;
                        end;
                end;
        end;
end;

// ˸ ø  Ȯ
function TKakaoService.ListATSTemplate(CorpNum : String; UserID : String = '') : TATSTemplateList;
var
        responseJson : String;
        jSons, btnjSons : ArrayOfString;
        i, j: Integer;

begin
        try
                responseJson := httpget('/KakaoTalk/ListATSTemplate', CorpNum, UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code,le.message);
                                exit;
                        end
                        else
                        begin
                                SetLength(result,0);
                                SetLength(jSons,0);
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                exit;
        end
        else
        begin

        try
                jSons := ParseJsonList(responseJson);
                SetLength(result,Length(jSons));

                for i:= 0 to Length(jSons)-1 do
                begin
                        result[i] := TATSTemplate.Create;
                        result[i].templateCode := getJsonString(jSons[i], 'templateCode');
                        result[i].templateName := getJsonString(jSons[i], 'templateName');
                        result[i].template := getJsonString(jSons[i], 'template');
                        result[i].plusFriendID := getJsonString(jSons[i], 'plusFriendID');
                        result[i].ads := getJsonString(jSons[i], 'ads');
                        result[i].appendix := getJsonString(jSons[i], 'appendix');
                        result[i].secureYN := getJsonBoolean(responseJson, 'secureYN');
                        result[i].state := StrtoInt(getJsonString(responseJson, 'state'));
                        result[i].stateDT := getJsonString(responseJson, 'stateDT');
                        // ˸ ø ư
                        btnjSons := getJSonList(jSons[i], 'btns');
                        SetLength(result[i].btns, Length(btnjSons));

                        for j:= 0 to Length(btnjSons)-1 do
                        begin
                                result[i].btns[j] := TSendKakaoButton.Create;
                                result[i].btns[j].buttonName := getJSonString(btnjSons[j],'n');
                                result[i].btns[j].buttonType := getJSonString(btnjSons[j],'t');
                                result[i].btns[j].buttonURL1 := getJSonString(btnjSons[j],'u1');
                                result[i].btns[j].buttonURL2 := getJSonString(btnjSons[j],'u2');
                        end;
                        SetLength(btnjSons,0);
                end;
        except
                on E:Exception do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(-99999999,'ó .[Malformed Json]');
                                exit;
                        end
                        else
                        begin
                                SetLength(result,0);
                                SetLength(jSons,0);
                                setLastErrCode(-99999999);
                                setLastErrMessage('ó .[Malformed Json]');
                                exit;
                        end;
                end;

        end;
        end;
end;

function TKakaoService.CheckSenderNumber(CorpNum : String; SenderNumber : String; UserID : String = '') : TResponse;
var
        responseJson : String;
begin
        if SenderNumber = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999,'߽Źȣ Էµ ʾҽϴ');
                        exit;
                end
                else
                begin
                        result.code := -99999999;
                        result.message := '߽Źȣ Էµ ʾҽϴ';
                        exit;
                end;
        end;
        
        try
                responseJson := httpget('/KakaoTalk/CheckSenderNumber/'+SenderNumber,CorpNum,UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.Message);
                                exit;
                        end
                        else
                        begin
                                result.code := le.code;
                                result.message := le.message;
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                result.code := LastErrCode;
                result.message := LastErrMessage;
                exit;
        end
        else
        begin
                result.code := getJsonInteger(responseJson,'code');
                result.message := getJsonString(responseJson,'message');
                exit;
        end;
end;

//  
function TKakaoService.CancelReserve(CorpNum : String; receiptNum : String; UserID : String = '') : TResponse;
var
        responseJson : String;
begin
        if receiptNum = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999,'ȣ Էµ ʾҽϴ');
                        exit;
                end
                else
                begin
                        result.code := -99999999;
                        result.message := 'ȣ Էµ ʾҽϴ';
                        exit;
                end;
        end;
        
        try
                responseJson := httpget('/KakaoTalk/'+receiptNum+'/Cancel',CorpNum,UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.Message);
                                exit;
                        end
                        else
                        begin
                                result.code := le.code;
                                result.message := le.message;
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                result.code := LastErrCode;
                result.message := LastErrMessage;
                exit;
        end
        else
        begin
                result.code := getJsonInteger(responseJson,'code');
                result.message := getJsonString(responseJson,'message');
                exit;
        end;
end;


function TKakaoService.GetMessages(CorpNum : String; receiptNum : String; UserID : String = '') : TSentKakaoInfo;
var
        responseJson : String;
        jSons : ArrayofString;
        i : Integer;
begin
        if receiptNum = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999,'ȣ Էµ ʾҽϴ');
                        exit;
                end
                else
                begin
                        result := TSentKakaoInfo.Create;
                        setLastErrCode(-99999999);
                        setLastErrMessage('ȣ Էµ ʾҽϴ');
                        exit;
                end;
        end;

        try
                responseJson := httpget('/KakaoTalk/'+receiptNum, CorpNum, UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code,le.message);
                                exit;
                        end
                        else
                        begin
                                result := TSentKakaoInfo.Create;
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                exit;
        end
        else
        begin
        try
                result := TSentKakaoInfo.Create;
                result.contentType := getJsonString(responseJson, 'contentType');
                result.templateCode := getJsonString(responseJson, 'templateCode');
                result.plusFriendID := getJsonString(responseJson, 'plusFriendID');
                result.sendNum := getJsonString(responseJson, 'sendNum');
                result.altSubject := getJsonString(responseJson, 'altSubject');
                result.altContent := getJsonString(responseJson, 'altContent');
                result.altSendType := getJsonString(responseJson, 'altSendType');
                result.reserveDT := getJsonString(responseJson, 'reserveDT');
                result.adsYN := getJsonBoolean(responseJson, 'adsYN');
                result.imageURL := getJsonString(responseJson, 'imageURL');
                result.sendCnt := getJsonString(responseJson, 'sendCnt');
                result.successCnt := getJsonString(responseJson, 'successCnt');
                result.failCnt := getJsonString(responseJson, 'failCnt');
                result.altCnt := getJsonString(responseJson, 'altCnt');
                result.cancelCnt := getJsonString(responseJson, 'cancelCnt');

                jSons := getJSonList(responseJson, 'btns');
                SetLength(result.btns, Length(jSons));

                for i:= 0 to Length(jSons)-1 do
                begin
                        result.btns[i] := TSendKakaoButton.Create;
                        result.btns[i].buttonName := getJSonString(jSons[i],'n');
                        result.btns[i].buttonType := getJSonString(jSons[i],'t');
                        result.btns[i].buttonURL1 := getJSonString(jSons[i],'u1');
                        result.btns[i].buttonURL2 := getJSonString(jSons[i],'u2');
                end;

                jSons := getJSonList(responseJson, 'msgs');
                SetLength(result.msgs, Length(jSons));

                for i:= 0 to Length(jSons)-1 do
                begin
                        result.msgs[i] := TSentKakaoDetail.Create;
                        result.msgs[i].state := getJsonInteger(jSons[i],'state');
                        result.msgs[i].sendDT := getJsonString(jSons[i],'sendDT');
                        result.msgs[i].result := getJsonInteger(jSons[i], 'result');
                        result.msgs[i].resultDT := getJsonString(jSons[i], 'resultDT');
                        result.msgs[i].contentType := getJsonString(jSons[i], 'contentType');
                        result.msgs[i].receiveNum := getJsonString(jSons[i], 'receiveNum');
                        result.msgs[i].receiveName := getJsonString(jSons[i], 'receiveName');
                        result.msgs[i].content := getJsonString(jSons[i], 'content');
                        result.msgs[i].altContentType := getJsonInteger(jSons[i], 'altContentType');
                        result.msgs[i].altSendDT := getJsonString(jSons[i], 'altSendDT');
                        result.msgs[i].altResult := getJsonInteger(jSons[i], 'altResult');
                        result.msgs[i].altResultDT := getJsonString(jSons[i], 'altResultDT');
                        result.msgs[i].altSubject := getJsonString(jSons[i], 'altSubject');
                        result.msgs[i].altContent := getJsonString(jSons[i], 'altContent');
                        result.msgs[i].receiptNum := getJsonString(jSons[i], 'receiptNum');
                        result.msgs[i].requestNum := getJsonString(jSons[i], 'requestNum');
                        result.msgs[i].interOPRefKey := getJsonString(jSons[i], 'interOPRefKey');                        
                end;

        except
                on E:Exception do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(-99999999, 'ó .[Malformed Json]');
                                exit;
                        end
                        else
                        begin
                                result := TSentKakaoInfo.Create();
                                setLastErrCode(-99999999);
                                setLastErrMessage('ó .[Malformed Json]');
                                exit;
                        end;
                end;
        end;
        end;
end;

function UrlEncodeUTF8(stInput : widestring) : string;
  const
    hex : array[0..255] of string = (
     '%00', '%01', '%02', '%03', '%04', '%05', '%06', '%07',
     '%08', '%09', '%0a', '%0b', '%0c', '%0d', '%0e', '%0f',
     '%10', '%11', '%12', '%13', '%14', '%15', '%16', '%17',
     '%18', '%19', '%1a', '%1b', '%1c', '%1d', '%1e', '%1f',
     '%20', '%21', '%22', '%23', '%24', '%25', '%26', '%27',
     '%28', '%29', '%2a', '%2b', '%2c', '%2d', '%2e', '%2f',
     '%30', '%31', '%32', '%33', '%34', '%35', '%36', '%37',
     '%38', '%39', '%3a', '%3b', '%3c', '%3d', '%3e', '%3f',
     '%40', '%41', '%42', '%43', '%44', '%45', '%46', '%47',
     '%48', '%49', '%4a', '%4b', '%4c', '%4d', '%4e', '%4f',
     '%50', '%51', '%52', '%53', '%54', '%55', '%56', '%57',
     '%58', '%59', '%5a', '%5b', '%5c', '%5d', '%5e', '%5f',
     '%60', '%61', '%62', '%63', '%64', '%65', '%66', '%67',
     '%68', '%69', '%6a', '%6b', '%6c', '%6d', '%6e', '%6f',
     '%70', '%71', '%72', '%73', '%74', '%75', '%76', '%77',
     '%78', '%79', '%7a', '%7b', '%7c', '%7d', '%7e', '%7f',
     '%80', '%81', '%82', '%83', '%84', '%85', '%86', '%87',
     '%88', '%89', '%8a', '%8b', '%8c', '%8d', '%8e', '%8f',
     '%90', '%91', '%92', '%93', '%94', '%95', '%96', '%97',
     '%98', '%99', '%9a', '%9b', '%9c', '%9d', '%9e', '%9f',
     '%a0', '%a1', '%a2', '%a3', '%a4', '%a5', '%a6', '%a7',
     '%a8', '%a9', '%aa', '%ab', '%ac', '%ad', '%ae', '%af',
     '%b0', '%b1', '%b2', '%b3', '%b4', '%b5', '%b6', '%b7',
     '%b8', '%b9', '%ba', '%bb', '%bc', '%bd', '%be', '%bf',
     '%c0', '%c1', '%c2', '%c3', '%c4', '%c5', '%c6', '%c7',
     '%c8', '%c9', '%ca', '%cb', '%cc', '%cd', '%ce', '%cf',
     '%d0', '%d1', '%d2', '%d3', '%d4', '%d5', '%d6', '%d7',
     '%d8', '%d9', '%da', '%db', '%dc', '%dd', '%de', '%df',
     '%e0', '%e1', '%e2', '%e3', '%e4', '%e5', '%e6', '%e7',
     '%e8', '%e9', '%ea', '%eb', '%ec', '%ed', '%ee', '%ef',
     '%f0', '%f1', '%f2', '%f3', '%f4', '%f5', '%f6', '%f7',
     '%f8', '%f9', '%fa', '%fb', '%fc', '%fd', '%fe', '%ff');
 var
   iLen,iIndex : integer;
   stEncoded : string;
   ch : widechar;
 begin
   iLen := Length(stInput);
   stEncoded := '';
   for iIndex := 1 to iLen do
   begin
     ch := stInput[iIndex];
     if (ch >= 'A') and (ch <= 'Z') then
       stEncoded := stEncoded + ch
     else if (ch >= 'a') and (ch <= 'z') then
       stEncoded := stEncoded + ch
     else if (ch >= '0') and (ch <= '9') then
       stEncoded := stEncoded + ch
     else if (ch = ' ') then
       stEncoded := stEncoded + '+'
     else if ((ch = '-') or (ch = '_') or (ch = '.') or (ch = '!') or (ch = '*')
       or (ch = '~') or (ch = '\')  or (ch = '(') or (ch = ')')) then
       stEncoded := stEncoded + ch
     else if (Ord(ch) <= $07F) then
       stEncoded := stEncoded + hex[Ord(ch)]
     else if (Ord(ch) <= $7FF) then
     begin
        stEncoded := stEncoded + hex[$c0 or (Ord(ch) shr 6)];
        stEncoded := stEncoded + hex[$80 or (Ord(ch) and $3F)];
     end
     else
     begin
        stEncoded := stEncoded + hex[$e0 or (Ord(ch) shr 12)];
        stEncoded := stEncoded + hex[$80 or ((Ord(ch) shr 6) and ($3F))];
        stEncoded := stEncoded + hex[$80 or ((Ord(ch)) and ($3F))];
     end;
   end;
   result := (stEncoded);
 end;

// ȸ
function TKakaoService.Search(CorpNum : String; SDate: String; EDate : String; State : Array Of String; Item : Array Of String; ReserveYN : String; SenderYN : boolean; Page : Integer; PerPage : Integer; Order : String; UserID : String = ''): TKakaoSearchList;
begin
        result := Search(CorpNum, SDate, EDate, State, Item, ReserveYN, SenderYN, Page, PerPage, Order, '', UserID);
end;

function TKakaoService.Search(CorpNum, SDate, EDate: String; State, Item: array of String; ReserveYN: String; SenderYN: boolean; Page, PerPage: Integer; Order, QString, UserID: String): TKakaoSearchList;
var
        responseJson : String;
        uri: String;
        StateList : String;
        ItemList : String;
        jSons : ArrayOfString;
        i : Integer;
begin

        for i := 0 to High(State)do
        begin
                if State[i] <> '' Then
                StateList := StateList + State[i] + ',';
        end;

        for i:= 0 to High(Item) do
        begin
                if Item[i] <> '' Then
                ItemList := ItemList + Item[i] + ',';
        end;

        uri := '/KakaoTalk/Search?SDate='+SDate+'&&EDate='+EDate;
        uri := uri + '&&State='+StateList;
        uri := uri + '&&Item='+ItemList;

        if Page < 1 then Page := 1;
        uri := uri + '&&Page='+IntToStr(Page);
        uri := uri + '&&PerPage=' + IntToStr(Perpage);

        if ReserveYN <> '' then uri := uri + '&&ReserveYN=' + ReserveYN;

        if SenderYN then uri := uri + '&&SenderYN=1'
        else uri := uri +'&&SenderYN=0';

        uri := uri + '&&Order=' + Order;

        if QString <> '' then uri := uri + '&&QString=' + UrlEncodeUTF8(QString);

        
        try
                responseJson := httpget(uri, CorpNum, UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code,le.message);
                                exit;
                        end
                        else
                        begin
                                result:= TKakaoSearchList.Create;
                                result.code := le.code;
                                result.message := le.message;
                                exit;
                        end;
                end;
        end;


        if LastErrCode <> 0 then
        begin
                result := TKakaoSearchList.Create;
                result.code := LastErrCode;
                result.message := LastErrMessage;
                exit;
        end
        else
        begin
        
        result := TKakaoSearchList.Create;

        result.code := getJsonInteger(responseJson, 'code');
        result.message := getJsonString(responseJson, 'message');
        result.total := getJsonInteger(responseJson, 'total');
        result.perPage := getJsonInteger(responseJson, 'perPage');
        result.pageNum := getJsonInteger(responseJson, 'pageNum');
        result.pageCount := getJsonInteger(responseJson, 'pageCount');

        try
                jSons := getJSonList(responseJson, 'list');
                SetLength(result.list, Length(jSons));

                for i:= 0 to Length(jSons)-1 do
                begin
                        result.list[i] := TSentKakaoDetail.Create;
                        result.list[i].state := getJsonInteger(jSons[i],'state');
                        result.list[i].sendDT := getJsonString(jSons[i],'sendDT');
                        result.list[i].result := getJsonInteger(jSons[i], 'result');
                        result.list[i].resultDT := getJsonString(jSons[i], 'resultDT');
                        result.list[i].contentType := getJsonString(jSons[i], 'contentType');
                        result.list[i].receiveNum := getJsonString(jSons[i], 'receiveNum');
                        result.list[i].receiveName := getJsonString(jSons[i], 'receiveName');
                        result.list[i].content := getJsonString(jSons[i], 'content');
                        result.list[i].altContentType := getJsonInteger(jSons[i], 'altContentType');
                        result.list[i].altSendDT := getJsonString(jSons[i], 'altSendDT');
                        result.list[i].altResult := getJsonInteger(jSons[i], 'altResult');
                        result.list[i].altResultDT := getJsonString(jSons[i], 'altResultDT');
                        result.list[i].altSubject := getJsonString(jSons[i], 'altSubject');
                        result.list[i].altContent := getJsonString(jSons[i], 'altContent');
                        result.list[i].receiptNum := getJsonString(jSons[i], 'receiptNum');
                        result.list[i].requestNum := getJsonString(jSons[i], 'requestNum');
                        result.list[i].interOPRefKey := getJsonString(jSons[i], 'interOPRefKey');                        
                end;
        except
                on E:Exception do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(-99999999,'ó .[Malformed Json]');
                                exit;
                        end
                        else
                        begin
                                result := TKakaoSearchList.Create;
                                result.code := -99999999;
                                result.message := 'ó .[Malformed Json]';
                                exit; 
                        end;
                end;
        end;
        end;
end;

// ˸ ܰ
function TKakaoService.SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; AltSendType : String; ReserveDT: String; ReceiverNum : String; ReceiverName : String; ATSMsg : String; ATSAltMsg : String; UserID : String = ''; requestNum : String = '') : String;
var
        Receivers : TSendKakaoReceiverList;
        Buttons : TSendKakaoButtonList;
begin
        SetLength(Receivers,1);
        Receivers[0] := TSendKakaoReceiver.Create;
        Receivers[0].rcv := ReceiverNum;
        Receivers[0].rcvnm := ReceiverName;
        Receivers[0].msg := ATSMsg;
        Receivers[0].altmsg := ATSAltMsg;

        SetLength(Buttons, 0);

        result := SendATS(CorpNum, TemplateCode, SenderNum, '', '', '', AltSendType, ReserveDT, Receivers, Buttons, UserID, requestNum);
end;

// ˸ ܰ - ư߰
function TKakaoService.SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; AltSendType : String; ReserveDT: String; ReceiverNum : String; ReceiverName : String; ATSMsg : String; ATSAltMsg : String; Buttons : TSendKakaoButtonList; UserID : String = ''; requestNum : String = '') : String;
var
        Receivers : TSendKakaoReceiverList;
begin
        SetLength(Receivers,1);
        Receivers[0] := TSendKakaoReceiver.Create;
        Receivers[0].rcv := ReceiverNum;
        Receivers[0].rcvnm := ReceiverName;
        Receivers[0].msg := ATSMsg;
        Receivers[0].altmsg := ATSAltMsg;

        result := SendATS(CorpNum, TemplateCode, SenderNum, '', '', '', AltSendType, ReserveDT, Receivers, Buttons, UserID, requestNum);
end;

// ˸ ܰ  - ü  ߰
function TKakaoService.SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; AltSendType : String; ReserveDT: String; ReceiverNum : String; ReceiverName : String; ATSMsg : String;  ATSAltSjt : String; ATSAltMsg : String; Buttons : TSendKakaoButtonList; UserID : String = ''; requestNum : String = '') : String;
var
        Receivers : TSendKakaoReceiverList;
begin
        SetLength(Receivers,1);
        Receivers[0] := TSendKakaoReceiver.Create;
        Receivers[0].rcv := ReceiverNum;
        Receivers[0].rcvnm := ReceiverName;
        Receivers[0].msg := ATSMsg;
        Receivers[0].altsjt := ATSAltSjt;
        Receivers[0].altmsg := ATSAltMsg;

        result := SendATS(CorpNum, TemplateCode, SenderNum, '', '', '', AltSendType, ReserveDT, Receivers, Buttons, UserID, requestNum);
end;

// ˸ 뷮
function TKakaoService.SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; Content : String; AltContent: String; AltSendType : String; ReserveDT: String; Receivers : TSendKakaoReceiverList; UserID : String = ''; requestNum : String = '') : String;
var
        Buttons : TSendKakaoButtonList;
begin
        SetLength(Buttons, 0);
        result := SendATS(CorpNum, TemplateCode, SenderNum, Content, '', AltContent, AltSendType, ReserveDT, Receivers, Buttons, UserID, requestNum); 

end;

// ˸ 뷮
function TKakaoService.SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; Content : String; AltContent: String; AltSendType : String; ReserveDT: String; Receivers : TSendKakaoReceiverList; Buttons : TSendKakaoButtonList; UserID : String = ''; requestNum : String = '' ) : String;
begin
        result := SendATS(CorpNum, TemplateCode, SenderNum, Content, '', AltContent, AltSendType, ReserveDT, Receivers, Buttons, UserID, requestNum); 
end;

// ˸ 뷮
function TKakaoService.SendATS(CorpNum : String; TemplateCode : String; SenderNum : String; Content : String; AltSubject : String; AltContent: String; AltSendType : String; ReserveDT: String; Receivers : TSendKakaoReceiverList; Buttons : TSendKakaoButtonList; UserID : String = ''; requestNum : String = '' ) : String;
var
         requestJson, responseJson : string;
         i,j  : Integer;
begin
        
        if TemplateCode = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999, '˸ øڵ尡 Էµ ʾҽϴ.');
                        exit;
                end
                else
                begin
                        result := '';
                        setLastErrCode(-99999999);
                        setLastErrMessage('˸ øڵ尡 Էµ ʾҽϴ.');
                        exit;                        
                end;
        end;
        
        if SenderNum = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999, '˸ ߽Źȣ Էµ ʾҽϴ.');
                        exit;
                end
                else
                begin
                        result := '';
                        setLastErrCode(-99999999);
                        setLastErrMessage('˸ ߽Źȣ Էµ ʾҽϴ.');
                        exit;
                end;
        end;
        
        if Length(Receivers) = 0 then
        begin
                if FIsThrowException then
                begin
                         raise EPopbillException.Create(-99999999, '迭 Էµ ʾҽϴ.');
                         exit;
                end
                else
                begin
                        result := '';
                        setLastErrCode(-99999999);
                        setLastErrMessage('迭 Էµ ʾҽϴ.');
                        exit;
                end;
        end;
        

        requestJson := '{';

        if TemplateCode <> ''  then requestJson := requestJson + '"templateCode":"' + EscapeString(TemplateCode) + '",';
        if SenderNum    <> ''  then requestJson := requestJson + '"snd":"'          + EscapeString(SenderNum)    + '",';
        if Content      <> ''  then requestJson := requestJson + '"content":"'      + EscapeString(Content)      + '",';
        if AltSubject   <> ''  then requestJson := requestJson + '"altSubject":"'   + EscapeString(AltSubject)   + '",';
        if AltContent   <> ''  then requestJson := requestJson + '"altContent":"'   + EscapeString(AltContent)   + '",';
        if AltSendType  <> ''  then requestJson := requestJson + '"altSendType":"'  + EscapeString(AltSendType)  + '",';
        if reserveDT    <> ''  then requestJson := requestJson + '"sndDT":"'        + EscapeString(reserveDT)    + '",';
        if requestNum   <> ''  then requestJson := requestJson + '"requestNum":"'   + EscapeString(requestNum)   + '",';

        requestJson := requestJson + '"msgs":[';
        for i := 0 to Length(Receivers) - 1 do begin
                requestJson := requestJson
                        + '{"rcv":"'   + EscapeString(Receivers[i].rcv)      + '",'
                        + '"rcvnm":"'  + EscapeString(Receivers[i].rcvnm)    + '",'
                        + '"msg":"'    + EscapeString(Receivers[i].msg)      + '",'
                        + '"interOPRefKey":"'    + EscapeString(Receivers[i].interOPRefKey)      + '",';

                if Length(Receivers[i].buttonList) > 0 then
                begin
                requestJson := requestJson + '"btns":[';
                for j := 0 to Length(Receivers[i].buttonList) - 1 do begin
                        requestJson := requestJson +
                               '{"n":"'+EscapeString(Receivers[i].buttonList[j].buttonName)+'",'+
                               '"t":"'+EscapeString(Receivers[i].buttonList[j].buttonType)+'",'+
                               '"u1":"'+EscapeString(Receivers[i].buttonList[j].buttonURl1)+'",'+
                               '"u2":"'+EscapeString(Receivers[i].buttonList[j].buttonURL2)+'",'+
                               '"tg":"'+EscapeString(Receivers[i].buttonList[j].buttonTG)+'"}';

                        if j < Length(Receivers[i].buttonList) - 1 then requestJson := requestJson + ',';
                end;

                requestJson := requestJson + '],';                
        end;
                requestJson := requestJson + '"altsjt":"' + EscapeString(Receivers[i].altsjt) +'",';
                requestJson := requestJson + '"altmsg":"' + EscapeString(Receivers[i].altmsg)   + '"}';
                if i < Length(Receivers) - 1 then requestJson := requestJson + ',';
        end;

        requestJson := requestJson + ']';
                
        if Length(Buttons) > 0 then
        begin
                requestJson := requestJson + ',"btns":[';
                for i := 0 to Length(Buttons) - 1 do begin
                        requestJson := requestJson +
                                '{"n":"'+EscapeString(Buttons[i].buttonName)+'",'+
                                '"t":"'+EscapeString(Buttons[i].buttonType)+'",'+
                                '"u1":"'+EscapeString(Buttons[i].buttonURl1)+'",'+
                                '"u2":"'+EscapeString(Buttons[i].buttonURL2)+'",'+
                                '"tg":"'+EscapeString(Buttons[i].buttonTG)+'"}';

                        if i < Length(Buttons) - 1 then requestJson := requestJson + ',';
                end;
                requestJson := requestJson + ']';                
        end;


        requestJson := requestJson + '}';

        try
                responseJson := httppost('/ATS', CorpNum, UserID, requestJson);
                result := getJsonString(responseJson, 'receiptNum');
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.message);
                                exit;
                        end;
                end;
        end;
end;

// ģ ؽƮ (ܰ)
function TKakaoService.SendFTS(CorpNum : String; PlusFriendID : String; SenderNum : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; ReceiverNum : String; ReceiverName : String; FTSMsg : String; FTSAltMsg : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String;
var
        Receivers : TSendKakaoReceiverList;
begin
        SetLength(Receivers,1);
        Receivers[0] := TSendKakaoReceiver.Create;
        Receivers[0].rcv := ReceiverNum;
        Receivers[0].rcvnm := ReceiverName;
        Receivers[0].msg := FTSMsg;
        Receivers[0].altmsg := FTSAltMsg;

        result := SendFTS(CorpNum, PlusFriendID, SenderNum, '', '', '', AltSendType, ReserveDT, AdsYN, Receivers, Buttons, UserID, requestNum);
end;

// ģ ؽƮ (ܰ) - ü  ߰
function TKakaoService.SendFTS(CorpNum : String; PlusFriendID : String; SenderNum : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; ReceiverNum : String; ReceiverName : String; FTSMsg : String; FTSAltSjt: STring; FTSAltMsg : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String;
var
        Receivers : TSendKakaoReceiverList;
begin
        SetLength(Receivers,1);
        Receivers[0] := TSendKakaoReceiver.Create;
        Receivers[0].rcv := ReceiverNum;
        Receivers[0].rcvnm := ReceiverName;
        Receivers[0].msg := FTSMsg;
        Receivers[0].altsjt := FTSAltSjt;
        Receivers[0].altmsg := FTSAltMsg;

        result := SendFTS(CorpNum, PlusFriendID, SenderNum, '', '', '', AltSendType, ReserveDT, AdsYN, Receivers, Buttons, UserID, requestNum);
end;

function TKakaoService.SendFTS(CorpNum : String; PlusFriendID : String; SenderNum : String; Content : String; AltContent : String; AltSendType : String; ReserveDt : String; AdsYN : Boolean; Receivers : TSendKakaoReceiverList; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String;
begin
        result := SendFTS(CorpNum, PlusFriendID, SenderNum, Content, '', AltContent, AltSendType, ReserveDT, AdsYN, Receivers, Buttons, UserID, requestNum);
end;

// ģ ؽƮ (뷮)
function TKakaoService.SendFTS(CorpNum : String; PlusFriendID : String; SenderNum : String; Content : String; AltSubject : String; AltContent : String; AltSendType : String; ReserveDt : String; AdsYN : Boolean; Receivers : TSendKakaoReceiverList; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String;
var
        requestJson, responseJson : string;
        i,j: Integer;
begin

        if SenderNum = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999, 'ģ ߽Źȣ Էµ ʾҽϴ.');
                        exit;
                end
                else
                begin
                        result := '';
                        setLastErrCode(-99999999);
                        setLastErrMessage('ģ ߽Źȣ Էµ ʾҽϴ.');
                        exit;
                end;
        end;

        if Length(Receivers) = 0 then
        begin
                if FIsThrowException then
                begin
                         raise EPopbillException.Create(-99999999, '迭 Էµ ʾҽϴ.');
                         exit;
                end
                else
                begin
                        result := '';
                        setLastErrCode(-99999999);
                        setLastErrMessage('迭 Էµ ʾҽϴ.');
                        exit;
                end;
        end;
        
        if PlusFriendID = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999, '÷ģ ̵ Էµ ʾҽϴ.');
                        exit;
                end
                else
                begin
                        result := '';
                        setLastErrCode(-99999999);
                        setLastErrMessage('÷ģ ̵ Էµ ʾҽϴ.');
                        exit;                        
                end;
        end;

        requestJson := '{';

        if PlusFriendID <> ''  then requestJson := requestJson + '"plusFriendID":"' + EscapeString(PlusFriendID) + '",';
        if SenderNum    <> ''  then requestJson := requestJson + '"snd":"'          + EscapeString(SenderNum)    + '",';
        if Content      <> ''  then requestJson := requestJson + '"content":"'      + EscapeString(Content)      + '",';
        if AltSubject   <> ''  then requestJson := requestJson + '"altSubject":"'   + EscapeString(AltSubject)   + '",';
        if AltContent   <> ''  then requestJson := requestJson + '"altContent":"'   + EscapeString(AltContent)   + '",';
        if AltSendType  <> ''  then requestJson := requestJson + '"altSendType":"'  + EscapeString(AltSendType)  + '",';
        if reserveDT    <> ''  then requestJson := requestJson + '"sndDT":"'        + EscapeString(reserveDT)    + '",';
        if requestNum   <> ''  then requestJson := requestJson + '"requestNum":"'   + EscapeString(requestNum)    + '",';
        if adsYN then requestJson := requestJson + '"adsYN":true,';

        requestJson := requestJson + '"msgs":[';
        for i := 0 to Length(Receivers) - 1 do begin
                requestJson := requestJson
                        + '{"rcv":"'   + EscapeString(Receivers[i].rcv)      + '",'
                        + '"rcvnm":"'  + EscapeString(Receivers[i].rcvnm)    + '",'
                        + '"msg":"'    + EscapeString(Receivers[i].msg)      + '",'
                        + '"interOPRefKey":"'    + EscapeString(Receivers[i].interOPRefKey)      + '",';

                if Length(Receivers[i].buttonList) > 0 then
                begin
                requestJson := requestJson + '"btns":[';
                for j := 0 to Length(Receivers[i].buttonList) - 1 do begin
                        requestJson := requestJson +
                               '{"n":"'+EscapeString(Receivers[i].buttonList[j].buttonName)+'",'+
                               '"t":"'+EscapeString(Receivers[i].buttonList[j].buttonType)+'",'+
                               '"u1":"'+EscapeString(Receivers[i].buttonList[j].buttonURl1)+'",'+
                               '"u2":"'+EscapeString(Receivers[i].buttonList[j].buttonURL2)+'"}';

                        if j < Length(Receivers[i].buttonList) - 1 then requestJson := requestJson + ',';
                end;

                requestJson := requestJson + '],';                
        end;
                requestJson := requestJson + '"altsjt":"' + EscapeString(Receivers[i].altsjt)+'",';
                requestJson := requestJson + '"altmsg":"' + EscapeString(Receivers[i].altmsg)   + '"}';
                if i < Length(Receivers) - 1 then requestJson := requestJson + ',';
        end;

        requestJson := requestJson + ']';

        requestJson := requestJson + ',"btns":[';
        for i := 0 to Length(Buttons) - 1 do begin
                requestJson := requestJson +
                        '{"n":"'+EscapeString(Buttons[i].buttonName)+'",'+
                        '"t":"'+EscapeString(Buttons[i].buttonType)+'",'+
                        '"u1":"'+EscapeString(Buttons[i].buttonURl1)+'",'+
                        '"u2":"'+EscapeString(Buttons[i].buttonURL2)+'"}';

                if i < Length(Buttons) - 1 then requestJson := requestJson + ',';
        end;
        requestJson := requestJson + ']';

        requestJson := requestJson + '}';

        try
                responseJson := httppost('/FTS', CorpNum, UserID, requestJson);
                result := getJsonString(responseJson, 'receiptNum');
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.message);
                                exit;
                        end;
                end;
        end;
end;


// ģ ̹ (ܰ)
function TKakaoService.SendFMS(CorpNum : String; PlusFriendID : String; SenderNum : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; ReceiverNum : String; ReceiverName : String; FTSMsg : String; FTSAltMsg : String; FMSFilePath : String; ImageURL : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String;
var
        Receivers : TSendKakaoReceiverList;
begin
        SetLength(Receivers,1);
        Receivers[0] := TSendKakaoReceiver.Create;
        Receivers[0].rcv := ReceiverNum;
        Receivers[0].rcvnm := ReceiverName;
        Receivers[0].msg := FTSMsg;
        Receivers[0].altmsg := FTSAltMsg;

        result := SendFMS(CorpNum, PlusFriendID, SenderNum, '', '', '', AltSendType, ReserveDT, AdsYN, Receivers, FMSFilePath, ImageURL, Buttons, UserID, requestNum);
end;

function TKakaoService.SendFMS(CorpNum : String; PlusFriendID : String; SenderNum : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; ReceiverNum : String; ReceiverName : String; FTSMsg : String; FTSAltSjt : String; FTSAltMsg : String; FMSFilePath : String; ImageURL : String; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String;
var
        Receivers : TSendKakaoReceiverList;
begin
        SetLength(Receivers,1);
        Receivers[0] := TSendKakaoReceiver.Create;
        Receivers[0].rcv := ReceiverNum;
        Receivers[0].rcvnm := ReceiverName;
        Receivers[0].msg := FTSMsg;
        Receivers[0].altsjt := FTSAltSjt;
        Receivers[0].altmsg := FTSAltMsg;

        result := SendFMS(CorpNum, PlusFriendID, SenderNum, '', '', '', AltSendType, ReserveDT, AdsYN, Receivers, FMSFilePath, ImageURL, Buttons, UserID, requestNum);
end;

function TKakaoService.SendFMS(CorpNum : String; PlusFriendID : String; SenderNum : String; Content : String; AltContent : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; Receivers : TSendKakaoReceiverList; FMSFilePath : String; ImageURl: string; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String;
begin
        result := SendFMS(CorpNum, PlusFriendID, SenderNum, Content, '', AltContent, AltSendType, ReserveDT, AdsYN, Receivers, FMSFilePath, ImageURL, Buttons, UserID, requestNum);
end;

// ģ ̹ (뷮)
function TKakaoService.SendFMS(CorpNum : String; PlusFriendID : String; SenderNum : String; Content : String; AltSubject : String; AltContent : String; AltSendType : String; ReserveDT : String; AdsYN : Boolean; Receivers : TSendKakaoReceiverList; FMSFilePath : String; ImageURl: string; Buttons : TSendKakaoButtonList; UserID : string = ''; requestNum : String = '') : String;
var
        requestJson, responseJson : string;
        i,j : Integer;
        files : TFileList;
begin
        if SenderNum = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999, 'ģ ߽Źȣ Էµ ʾҽϴ.');
                        exit;
                end
                else
                begin
                        result := '';
                        setLastErrCode(-99999999);
                        setLastErrMessage('ģ ߽Źȣ Էµ ʾҽϴ.');
                        exit;
                end;
        end;

        if Length(Receivers) = 0 then
        begin
                if FIsThrowException then
                begin
                         raise EPopbillException.Create(-99999999, '迭 Էµ ʾҽϴ.');
                         exit;
                end
                else
                begin
                        result := '';
                        setLastErrCode(-99999999);
                        setLastErrMessage('迭 Էµ ʾҽϴ.');
                        exit;
                end;
        end;
        
        if PlusFriendID = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999, '÷ģ ̵ Էµ ʾҽϴ.');
                        exit;
                end
                else
                begin
                        result := '';
                        setLastErrCode(-99999999);
                        setLastErrMessage('÷ģ ̵ Էµ ʾҽϴ.');
                        exit;                        
                end;
        end;

        SetLength(files,1);

        files[0] := TFile.Create;
        files[0].FieldName := 'file';                 
        files[0].FileName := ExtractFileName(FMSFilePath);
        files[0].Data := TFileStream.Create(FMSFilePath,fmOpenRead);

        requestJson := '{';

        if PlusFriendID <> ''          then requestJson := requestJson + '"plusFriendID":"' + EscapeString(PlusFriendID) + '",';
        if SenderNum <> ''          then requestJson := requestJson + '"snd":"' + EscapeString(SenderNum) + '",';
        if Content <> ''         then requestJson := requestJson + '"content":"' + EscapeString(Content) + '",';
        if AltSubject <> ''         then requestJson := requestJson + '"altSubject":"' + EscapeString(AltSubject) + '",';
        if AltContent <> ''         then requestJson := requestJson + '"altContent":"' + EscapeString(AltContent) + '",';
        if AltSendType <> ''         then requestJson := requestJson + '"altSendType":"' + EscapeString(AltSendType) + '",';
        if reserveDT <> ''       then requestJson := requestJson + '"sndDT":"' + EscapeString(reserveDT) + '",';
        if imageURL <> ''       then requestJson := requestJson + '"imageURL":"' + EscapeString(imageURL) + '",';
        if requestNum <> ''       then requestJson := requestJson + '"requestNum":"' + EscapeString(requestNum) + '",';
        if adsYN then requestJson := requestJson + '"adsYN":true,';

       requestJson := requestJson + '"msgs":[';
        for i := 0 to Length(Receivers) - 1 do begin
                requestJson := requestJson
                        + '{"rcv":"'   + EscapeString(Receivers[i].rcv)      + '",'
                        + '"rcvnm":"'  + EscapeString(Receivers[i].rcvnm)    + '",'
                        + '"msg":"'    + EscapeString(Receivers[i].msg)      + '",'
                        + '"interOPRefKey":"'    + EscapeString(Receivers[i].interOPRefKey)      + '",';

                if Length(Receivers[i].buttonList) > 0 then
                begin
                requestJson := requestJson + '"btns":[';
                for j := 0 to Length(Receivers[i].buttonList) - 1 do begin
                        requestJson := requestJson +
                               '{"n":"'+EscapeString(Receivers[i].buttonList[j].buttonName)+'",'+
                               '"t":"'+EscapeString(Receivers[i].buttonList[j].buttonType)+'",'+
                               '"u1":"'+EscapeString(Receivers[i].buttonList[j].buttonURl1)+'",'+
                               '"u2":"'+EscapeString(Receivers[i].buttonList[j].buttonURL2)+'"}';

                        if j < Length(Receivers[i].buttonList) - 1 then requestJson := requestJson + ',';
                end;

                requestJson := requestJson + '],';                
        end;
                requestJson := requestJson + '"altsjt":"' + EscapeString(Receivers[i].altsjt)+'",';
                requestJson := requestJson + '"altmsg":"' + EscapeString(Receivers[i].altmsg)   + '"}';
                if i < Length(Receivers) - 1 then requestJson := requestJson + ',';
        end;

        requestJson := requestJson + ']';

        requestJson := requestJson + ',"btns":[';
        for i := 0 to Length(Buttons) - 1 do begin
                requestJson := requestJson +
                        '{"n":"'+EscapeString(Buttons[i].buttonName)+'",'+
                        '"t":"'+EscapeString(Buttons[i].buttonType)+'",'+
                        '"u1":"'+EscapeString(Buttons[i].buttonURl1)+'",'+
                        '"u2":"'+EscapeString(Buttons[i].buttonURL2)+'"}';

                if i < Length(Buttons) - 1 then requestJson := requestJson + ',';
        end;
        requestJson := requestJson + ']';

        requestJson := requestJson + '}';

        try
                try
                        responseJson := httppost('/FMS', CorpNum, UserID, requestJson, files);
                        result := getJsonString(responseJson, 'receiptNum');                        
                except
                        on le : EPopbillException do begin
                                if FIsThrowException then
                                begin
                                        raise EPopbillException.Create(le.code, le.message);
                                        exit;
                                end;
                        end;
                end;
        finally
                for i := 0 to Length(files) -1 do
                begin
                        files[i].Data.Free;
                end;
        end;

end;


// ۴ܰ ȸ
function TKakaoService.GetUnitCost(CorpNum : String; KakaoMsgType:EnumKakaoType; UserID : String = '') : Single;
var
        responseJson : String;
begin

        try 
                responseJson := httpget('/KakaoTalk/UnitCost?Type='+GetEnumName(TypeInfo(EnumKakaoType),integer(KakaoMsgType)), CorpNum, UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.message);
                                exit;
                        end
                        else
                        begin
                                result := 0.0;
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                exit;
        end
        else
        begin
                result := strToFloat(getJsonString(responseJson, 'unitCost'));
        end;
end;

//  ȸ
function TKakaoService.GetChargeInfo(CorpNum : String; KakaoMsgType:EnumKakaoType; UserID : String = '') : TKakaoChargeInfo;
var
        responseJson : String;
begin
        try
                responseJson := httpget('/KakaoTalk/ChargeInfo?Type='+GetEnumName(TypeInfo(EnumKakaoType),integer(KakaoMsgType)), CorpNum, UserID);
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code,le.message);
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                exit;
        end
        else
        begin

        try
                result := TKakaoChargeInfo.Create;

                result.unitCost := getJSonString(responseJson, 'unitCost');
                result.chargeMethod := getJSonString(responseJson, 'chargeMethod');
                result.rateSystem := getJSonString(responseJson, 'rateSystem');

        except
                on E:Exception do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(-99999999,'ó .[Malformed Json]');
                                exit;
                        end
                        else
                        begin
                                result := TKakaoChargeInfo.Create;
                                setLastErrCode(-99999999);
                                setLastErrMessage('ó .[Malformed Json]');
                                exit;
                        end;
                end;
        end;
        end;
end;

function TKakaoService.GetMessagesRN(CorpNum, requestNum, UserID: String): TSentKakaoInfo;
var
        responseJson : String;
        jSons : ArrayofString;
        i : Integer;
begin
        if requestNum = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999,'ûȣ(requestNum) Էµ ʾҽϴ');
                        exit;
                end
                else
                begin
                        result := TSentKakaoInfo.Create;
                        setLastErrCode(-99999999);
                        setLastErrMessage('ûȣ(requestNum) Էµ ʾҽϴ');
                        exit;
                end;
        end;

        try

        responseJson := httpget('/KakaoTalk/Get/'+ requestNum, CorpNum, UserID);

        try
                result := TSentKakaoInfo.Create;
                result.contentType := getJsonString(responseJson, 'contentType');
                result.templateCode := getJsonString(responseJson, 'templateCode');
                result.plusFriendID := getJsonString(responseJson, 'plusFriendID');
                result.sendNum := getJsonString(responseJson, 'sendNum');
                result.altSubject := getJsonString(responseJson, 'altSubject');
                result.altContent := getJsonString(responseJson, 'altContent');
                result.altSendType := getJsonString(responseJson, 'altSendType');
                result.reserveDT := getJsonString(responseJson, 'reserveDT');
                result.adsYN := getJsonBoolean(responseJson, 'adsYN');
                result.imageURL := getJsonString(responseJson, 'imageURL');
                result.sendCnt := getJsonString(responseJson, 'sendCnt');
                result.successCnt := getJsonString(responseJson, 'successCnt');
                result.failCnt := getJsonString(responseJson, 'failCnt');
                result.altCnt := getJsonString(responseJson, 'altCnt');
                result.cancelCnt := getJsonString(responseJson, 'cancelCnt');

                jSons := getJSonList(responseJson, 'btns');
                SetLength(result.btns, Length(jSons));

                for i:= 0 to Length(jSons)-1 do
                begin
                        result.btns[i] := TSendKakaoButton.Create;
                        result.btns[i].buttonName := getJSonString(jSons[i],'n');
                        result.btns[i].buttonType := getJSonString(jSons[i],'t');
                        result.btns[i].buttonURL1 := getJSonString(jSons[i],'u1');
                        result.btns[i].buttonURL2 := getJSonString(jSons[i],'u2');
                end;

                jSons := getJSonList(responseJson, 'msgs');
                SetLength(result.msgs, Length(jSons));

                for i:= 0 to Length(jSons)-1 do
                begin
                        result.msgs[i] := TSentKakaoDetail.Create;
                        result.msgs[i].state := getJsonInteger(jSons[i],'state');
                        result.msgs[i].sendDT := getJsonString(jSons[i],'sendDT');
                        result.msgs[i].result := getJsonInteger(jSons[i], 'result');
                        result.msgs[i].resultDT := getJsonString(jSons[i], 'resultDT');
                        result.msgs[i].contentType := getJsonString(jSons[i], 'contentType');
                        result.msgs[i].receiveNum := getJsonString(jSons[i], 'receiveNum');
                        result.msgs[i].receiveName := getJsonString(jSons[i], 'receiveName');
                        result.msgs[i].content := getJsonString(jSons[i], 'content');
                        result.msgs[i].altContentType := getJsonInteger(jSons[i], 'altContentType');
                        result.msgs[i].altSendDT := getJsonString(jSons[i], 'altSendDT');
                        result.msgs[i].altResult := getJsonInteger(jSons[i], 'altResult');
                        result.msgs[i].altResultDT := getJsonString(jSons[i], 'altResultDT');
                        result.msgs[i].altSubject := getJsonString(jSons[i], 'altSubject');
                        result.msgs[i].altContent := getJsonString(jSons[i], 'altContent');
                        result.msgs[i].receiptNum := getJsonString(jSons[i], 'receiptNum');
                        result.msgs[i].requestNum := getJsonString(jSons[i], 'requestNum');
                        result.msgs[i].interOPRefKey := getJsonString(jSons[i], 'interOPRefKey');
                      
                end;

                except
                        on E:Exception do begin
                                if FIsThrowException then
                                begin
                                        raise EPopbillException.Create(-99999999, 'ó .[Malformed Json]');
                                        exit;
                                end
                                else
                                begin
                                        result := TSentKakaoInfo.Create;
                                        setLastErrCode(-99999999);
                                        setLastErrMessage('ó .[Malformed Json]');
                                        exit;
                                end;
                        end;
                end;
        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code,le.message);
                        end
                        else
                        begin
                                result := TSentKakaoInfo.Create();
                                exit;
                        end;
                end;
        end;
end;

function TKakaoService.CancelReserveRN(CorpNum, requestNum, UserID: String): TResponse;
var
        responseJson : String;
begin
        if requestNum = '' then
        begin
                if FIsThrowException then
                begin
                        raise EPopbillException.Create(-99999999,'ûȣ(requestNum) Էµ ʾҽϴ');
                        exit;
                end
                else
                begin
                        result.code := -99999999;
                        result.message := 'ûȣ(requestNum) Էµ ʾҽϴ'; 
                        exit;
                end;
        end;

        try
                responseJson := httpget('/KakaoTalk/Cancel/' + requestNum, CorpNum, UserID);


        except
                on le : EPopbillException do begin
                        if FIsThrowException then
                        begin
                                raise EPopbillException.Create(le.code, le.Message);
                                exit;
                        end
                        else
                        begin
                                result.code := le.code;
                                result.message := le.message;
                                exit;
                        end;
                end;
        end;

        if LastErrCode <> 0 then
        begin
                result.code := LastErrCode;
                result.message := LastErrMessage;
        end
        else
        begin
                result.code := getJsonInteger(responseJson,'code');
                result.message := getJsonString(responseJson,'message');
        end;
end;


// End of Unit
end.
